/**
 * @file cs1d.cpp
 * @brief 
 *
 */

#include "cs1d.h"
#include "knapsack.h"
#include <cmath>
#include <iostream>
#include <fstream>

const double EPS = 1e-5;

void Data::read(const char* filename)
{
	std::ifstream in(filename);
	// read data from file
	in >> m; // numero di misure
	in >> W; // lunghezza dei tondini
	std::cout << "m = " << m << std::endl;
	std::cout << "W = " << W << std::endl;
	for (int i = 0; i < m; i++) //per ogni misura
	{
		double l;
		double r;
		in >> l;	//lunghezza
		in >> r;	//richiesta
		L.push_back(l);
		R.push_back(r);
		std::cout << l << " " << r << std::endl;
	}
	in.close();
}

void CS1D::initMaster(const Data& data)
{
	// setup initial LP
	std::vector<double> obj(data.m, 1.0);
	CHECKED_CPX_CALL( CPXnewcols, env, lp, data.m, &obj[0], 0, 0, 0, 0 );
	std::vector<int> matbeg(data.m);
	std::vector<int> idx(data.m);
	std::vector<double> coef(data.m);
	std::vector<char> sense(data.m, 'G');
	for (int i = 0; i < data.m; i++)
	{
		matbeg[i] = i;
		idx[i] = i;
		coef[i] = floor(data.W / data.L[i]);
	}
	CHECKED_CPX_CALL( CPXaddrows, env, lp, 0, data.m, data.m, &(data.R[0]), &sense[0], &matbeg[0], &idx[0], &coef[0], 0, 0 );
	
	// debug
	CHECKED_CPX_CALL( CPXwriteprob, env, lp, "initial.lp", NULL );
}

void CS1D::solveMasterLP(std::vector<double>& x, std::vector<double>& u, double& objval)
{
	// solve
	CHECKED_CPX_CALL( CPXlpopt, env, lp );
	// get primal and dual solutions
	CHECKED_CPX_CALL( CPXgetobjval, env, lp, &objval );
	int n = CPXgetnumcols(env, lp);
	x.resize(n);
	CHECKED_CPX_CALL( CPXgetx, env, lp, &x[0], 0, n - 1 );
	int m = CPXgetnumrows(env, lp);
	u.resize(m);
	CHECKED_CPX_CALL( CPXgetpi, env, lp, &u[0], 0, m - 1 );
}

bool CS1D::price(Env pricerEnv, const Data& data, const std::vector<double>& u)
{
	KPSolver kp(pricerEnv);
	std::vector<double> p;
	double value;
	kp.exec(data.L, u, data.W, p, value);
	if (value < (1.0 + EPS)) return false;
	// add column
	std::vector<int> idx;
	std::vector<double> coef;
	int m = p.size();
	for(int i = 0; i < m; i++)
	{
		if (p[i] > EPS)
		{
			idx.push_back(i);
			coef.push_back(p[i]);
		}
	}
	double obj = 1.0;
	int matbeg = 0;
	CHECKED_CPX_CALL( CPXaddcols, env, lp, 1, idx.size(), &obj, &matbeg, &idx[0], &coef[0], NULL, NULL, NULL );
	return true;
}

void CS1D::branchAndCut(std::vector<double>& x, double& objval)
{
	// change to MIP
	int n = CPXgetnumcols(env, lp);
	std::vector<char> xtype(n, 'I');
	CHECKED_CPX_CALL( CPXcopyctype, env, lp, &xtype[0] );
	// solve
	CHECKED_CPX_CALL( CPXmipopt, env, lp );
	// get solution
	CHECKED_CPX_CALL( CPXgetobjval, env, lp, &objval );
	x.resize(n);
	CHECKED_CPX_CALL( CPXgetx, env, lp, &x[0], 0, n - 1 );
}

